package it.unimi.dsi.util;

/*		 
 * DSI utilities
 *
 * Copyright (C) 2011-2012 Sebastiano Vigna 
 *
 *  This library is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU Lesser General Public License as published by the Free
 *  Software Foundation; either version 3 of the License, or (at your option)
 *  any later version.
 *
 *  This library is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program; if not, see <http://www.gnu.org/licenses/>.
 *
 */


import it.unimi.dsi.Util;

import java.util.Random;

/** A fast, high-quality 64-bit pseudorandom number generator that combines George Marsaglia's Xorshift
 * generators (described in <a href="http://www.jstatsoft.org/v08/i14/paper/">&ldquo;Xorshift RNGs&rdquo;</a>,
 * <i>Journal of Statistical Software</i>, 8:1&minus;6, 2003) with a multiplication.
 * Note that this is <strong>not</strong> a cryptographic-strength
 * pseudorandom number generator, but its quality is preposterously higher than {@link Random}'s 
 * (and its cycle length is 2<sup>64</sup>&nbsp;&minus;&nbsp;1).
 * 
 * <p>On an Intel i5, calls to {@link #nextLong(long)} range from 20% faster to fifty times 
 * faster than {@link java.util.concurrent.ThreadLocalRandom}'s, and 
 * calls to {@link #nextInt(int)} range from 20% slower to two times faster, depending on the argument.
 * Timings are orders of magnitude faster than {@link Random}'s, but {@link Random} is slowed down by the
 * fact of being thread safe, so the comparison is not fair.
 * 
 * <p>This class extends {@link Random}, overriding (as usual) the {@link Random#next(int)} method. Nonetheless,
 * since the generator is inherently 64-bit also {@link Random#nextInt()}, {@link Random#nextInt(int)},
 * {@link Random#nextLong()} and {@link Random#nextDouble()} have been overridden for speed (preserving, of course, {@link Random}'s semantics).
 * 
 * <h3>Parameter choice</h3>
 * 
 * <p>There are five parameters to choose in a pseudorandom number generator of this kind: the three shift values,
 * the type of shift, and the multiplier. <i>Numerical Recipes</i> (third edition, Cambridge University Press, 2007) 
 * suggests a choice of parameters from
 * which we take only the multiplier (actually, proposed by Pierre L'Ecuyer
 * in &ldquo;Tables of linear congruential generators of different sizes and good lattice structure&rdquo;,
 * <i>Math. Comput.</i>, 68(225):249&minus;260, 1999). The remaining parameters have been set following
 * extensive experimentation on the 2200 possible choices using 
 * <a href="http://www.iro.umontreal.ca/~simardr/testu01/tu01.html">TestU01</a> and
 * <a href="http://www.phy.duke.edu/~rgb/General/dieharder.php">Dieharder</a>. More details
 * will appear in a forthcoming paper.
 * 
 * <h3>Notes</h3>
 * 
 * <p>The <em>lower bits</em> of this generator are of higher quality than the upper bits. Thus, masking the lower
 * bits is a safe and effective way to obtain small random numbers. The code in this class methodically extracts
 * lower bits, rather than upper bits, whenever a subset of bits is necessary.
 *  
 */
public class XorShiftStarRandom extends Random {
	private static final long serialVersionUID = 1L;

	/** 2<sup>53</sup> &minus; 1. */
	private static final long DOUBLE_MASK = ( 1L << 53 ) - 1;
	/** 2<sup>-53</sup>. */
	private static final double NORM_53 = 1. / ( 1L << 53 );
	/** 2<sup>24</sup> &minus; 1. */
	private static final long FLOAT_MASK = ( 1L << 24 ) - 1;
	/** 2<sup>-24</sup>. */
	private static final double NORM_24 = 1. / ( 1L << 24 );
	/** 2<sup>62</sup> &minus; 1. */
	private static final long NONNEGATIVE_LONG_MASK = ( 1L << 62 ) - 1;

	/** The internal state (and last returned value) of the algorithm. */
	private long x;

	public XorShiftStarRandom() {
		this( Util.randomSeed() );
	}

	/** Creates a new generator using a given seed.
	 * 
	 * @param seed a nonzero seed for the generator (if zero, the generator will be seeded with -1).
	 */
	public XorShiftStarRandom( final long seed ) {
		setSeed( seed );
		nextLong(); // Warm-up.
	}

	@Override
	protected int next( int bits ) {
		return (int)( nextLong() & ( 1L << bits ) - 1 );
	}

	@Override
	public long nextLong() {
		x ^= x << 23;
		x ^= x >>> 52;
		return 2685821657736338717L * ( x ^= ( x >>> 17 ) );
	}

	@Override
	public int nextInt() {
		return (int)nextLong();
	}
	
	@Override
	public int nextInt( final int n ) {
        if ( n <= 0 ) throw new IllegalArgumentException();
        for(;;) {
        	final long bits = nextLong() & NONNEGATIVE_LONG_MASK;
        	final long value = bits % n;
        	if ( bits - value + ( n - 1 ) >= 0 ) return (int)value;
        }
	}
	
	public long nextLong( final long n ) {
        if ( n <= 0 ) throw new IllegalArgumentException();
		// No special provision for n power of two: all our bits are good.
		for(;;) {
			final long bits = nextLong() & NONNEGATIVE_LONG_MASK;
			final long value = bits % n;
			if ( bits - value + ( n - 1 ) >= 0 ) return value;
		}
	}
	
	@Override
	 public double nextDouble() {
		return ( nextLong() & DOUBLE_MASK ) * NORM_53;
	}
	
	@Override
	public float nextFloat() {
		return (float)( ( nextLong() & FLOAT_MASK ) * NORM_24 );
	}

	@Override
	public boolean nextBoolean() {
		return ( nextLong() & 1 ) != 0;
	}
	
	@Override
	public void nextBytes( final byte[] bytes ) {
		int i = bytes.length, n = 0;
		while( i != 0 ) {
			n = Math.min( i, 8 );
			for ( long bits = nextLong(); n-- != 0; bits >>= 8 ) bytes[ --i ] = (byte)bits;
		}
	}

	@Override
	public void setSeed( final long seed ) {
		x = seed == 0 ? -1 : seed;
	}
}
